import { isBoolean } from "@thi.ng/checks/is-boolean";
import { isNumber } from "@thi.ng/checks/is-number";
import { M2, M3, M4, V2, V3, V4, } from "../api/types.js";
import { isVec } from "./checks.js";
export const lit = (type, val, info) => type === val.type && info === val.info
    ? val
    : {
        tag: "lit",
        type,
        info,
        val,
    };
export const bool = (x) => lit("bool", isNumber(x) ? !!x : x);
export const float = (x) => lit("float", isBoolean(x) ? x & 1 : x);
export const int = (x) => lit("int", isBoolean(x) ? x & 1 : isNumber(x) ? x | 0 : x);
export const uint = (x) => lit("uint", isBoolean(x) ? x & 1 : isNumber(x) ? x >>> 0 : x);
const wrap = (type, ctor) => (x) => isNumber(x)
    ? ctor(x)
    : x !== undefined && !isVec(x) && x.type !== type
        ? ctor(x)
        : x;
/**
 * Takes a plain number or numeric term and wraps it as float literal if
 * needed.
 *
 * @param x -
 */
export const wrapFloat = wrap("float", float);
/**
 * Takes a plain number or numeric term and wraps it as signed integer
 * literal if needed.
 *
 * @param x -
 */
export const wrapInt = wrap("int", int);
/**
 * Takes a plain number or numeric term and wraps it as unsigned integer
 * literal if needed.
 *
 * @param x -
 */
export const wrapUint = wrap("uint", uint);
/**
 * Takes a plain number or numeric term and wraps it as boolean literal
 * if needed.
 *
 * @param x -
 */
export const wrapBool = wrap("bool", bool);
export const TRUE = bool(true);
export const FALSE = bool(false);
export const FLOAT0 = float(0);
export const FLOAT1 = float(1);
export const FLOAT2 = float(2);
export const FLOAT05 = float(0.5);
export const INT0 = int(0);
export const INT1 = int(1);
export const UINT0 = uint(0);
export const UINT1 = uint(1);
export const PI = float(Math.PI);
export const TAU = float(Math.PI * 2);
export const HALF_PI = float(Math.PI / 2);
export const SQRT2 = float(Math.SQRT2);
export const PHI = float((1 + Math.sqrt(5)) / 2);
const $gvec = (wrap, init) => (xs) => [xs[0] === undefined ? init : wrap(xs[0]), ...xs.slice(1).map(wrap)];
const $vec = $gvec(wrapFloat, FLOAT0);
const $ivec = $gvec(wrapInt, INT0);
const $uvec = $gvec(wrapUint, UINT0);
const $bvec = $gvec(wrapBool, FALSE);
const $vinfo = (v, info = "") => v[0] + info.substring(1);
const $info = (xs, info) => isVec(xs[0]) ? $vinfo(xs[0].type, info[xs.length]) : info[xs.length];
const $gvec2 = (type, ctor, xs) => lit(type, (xs = ctor(xs)), $info(xs, ["n", "n"]));
const $gvec3 = (type, ctor, xs) => lit(type, (xs = ctor(xs)), $info(xs, ["n", "n", "vn"]));
const $gvec4 = (type, ctor, xs) => lit(type, (xs = ctor(xs)), xs.length === 2
    ? isVec(xs[1])
        ? xs[0].type[0] + xs[1].type[0]
        : "vn"
    : $info(xs, ["n", "n", , "vnn"]));
const $gmat = (type, info, xs) => lit(type, (xs = $vec(xs)), info[xs.length]);
// prettier-ignore
export function vec2(...xs) {
    return $gvec2(V2, $vec, xs);
}
export function vec3(...xs) {
    return $gvec3(V3, $vec, xs);
}
export function vec4(...xs) {
    return $gvec4(V4, $vec, xs);
}
// prettier-ignore
export function ivec2(...xs) {
    return $gvec2("ivec2", $ivec, xs);
}
export function ivec3(...xs) {
    return $gvec3("ivec3", $ivec, xs);
}
export function ivec4(...xs) {
    return $gvec4("ivec4", $ivec, xs);
}
// prettier-ignore
export function uvec2(...xs) {
    return $gvec2("uvec2", $uvec, xs);
}
export function uvec3(...xs) {
    return $gvec3("uvec3", $uvec, xs);
}
export function uvec4(...xs) {
    return $gvec4("uvec4", $uvec, xs);
}
// prettier-ignore
export function bvec2(...xs) {
    return $gvec2("bvec2", $bvec, xs);
}
export function bvec3(...xs) {
    return $gvec3("bvec3", $bvec, xs);
}
export function bvec4(...xs) {
    return $gvec4("bvec4", $bvec, xs);
}
export function mat2(...xs) {
    return $gmat(M2, ["n", "n", "vv"], xs);
}
export function mat3(...xs) {
    return $gmat(M3, ["n", "n", , "vvv"], xs);
}
export function mat4(...xs) {
    return $gmat(M4, ["n", "n", , , "vvvv"], xs);
}
export const VEC2_0 = vec2(0);
export const VEC2_1 = vec2(1);
export const VEC2_2 = vec2(2);
export const VEC3_0 = vec3(0);
export const VEC3_1 = vec3(1);
export const VEC3_2 = vec3(2);
