import { isNumber } from "@thi.ng/checks/is-number";
import { isMat, isVec } from "./checks.js";
import { numberWithMatchingType } from "./item.js";
import { float } from "./lit.js";
export const op1 = (op, val, post = false) => ({
    tag: "op1",
    type: val.type,
    op,
    val,
    post,
});
const OP_INFO = {
    mave: "mv",
    vema: "vm",
    vefl: "vn",
    mafl: "vn",
    flve: "nv",
    flma: "nv",
    ivin: "vn",
    iniv: "nv",
    uvui: "vn",
    uiuv: "nv",
};
export const op2 = (op, _l, _r, rtype, info) => {
    const nl = isNumber(_l);
    const nr = isNumber(_r);
    let type;
    let l;
    let r;
    if (nl) {
        if (nr) {
            // (number, number)
            l = float(_l);
            r = float(_r);
            type = "float";
        }
        else {
            // (number, term)
            r = _r;
            l = numberWithMatchingType(r, _l);
            type = r.type;
        }
    }
    else if (nr) {
        // (term, number)
        l = _l;
        r = numberWithMatchingType(l, _r);
        type = l.type;
    }
    else {
        // (term, term)
        l = _l;
        r = _r;
        type =
            rtype ||
                (isVec(l)
                    ? l.type
                    : isVec(r)
                        ? r.type
                        : isMat(r)
                            ? r.type
                            : l.type);
    }
    return {
        tag: "op2",
        type: rtype || type,
        info: info || OP_INFO[l.type.substring(0, 2) + r.type.substring(0, 2)],
        op,
        l: l,
        r: r,
    };
};
export const inc = (t) => op1("++", t, true);
export const dec = (t) => op1("--", t, true);
// prettier-ignore
export function add(l, r) {
    return op2("+", l, r);
}
export function sub(l, r) {
    return op2("-", l, r);
}
export function mul(l, r) {
    return op2("*", l, r, !isNumber(l) && !isNumber(r) && isMat(l) && isVec(r)
        ? r.type
        : undefined);
}
export function div(l, r) {
    return op2("/", l, r);
}
export function modi(l, r) {
    return op2("%", isNumber(l) ? numberWithMatchingType(r, l) : l, isNumber(r) ? numberWithMatchingType(l, r) : r);
}
/**
 * Syntax sugar for `-x`.
 *
 * @param val -
 */
export const neg = (val) => op1("-", val);
/**
 * Syntax sugar for `1 / x`.
 *
 * @param val -
 */
export const reciprocal = (val) => op2("/", 1, val);
// prettier-ignore
export function madd(a, b, c) {
    return add(mul(a, b), c);
}
// prettier-ignore
export function addm(a, b, c) {
    return mul(add(a, b), c);
}
export const not = (val) => op1("!", val);
export const or = (a, b) => op2("||", a, b);
export const and = (a, b) => op2("&&", a, b);
const cmp = (op) => (a, b) => op2(op, a, b, "bool");
export const eq = cmp("==");
export const neq = cmp("!=");
export const lt = cmp("<");
export const lte = cmp("<=");
export const gt = cmp(">");
export const gte = cmp(">=");
export const bitnot = (val) => op1("~", val);
// prettier-ignore
export function bitand(l, r) {
    return op2("&", l, r, undefined);
}
// prettier-ignore
export function bitor(l, r) {
    return op2("|", l, r, undefined);
}
// prettier-ignore
export function bitxor(l, r) {
    return op2("^", l, r, undefined);
}
