import { builtinCall } from "../ast/function.js";
import { matchingBoolType, matchingPrimFor } from "../ast/item.js";
const primOp1 = (name) => (a) => builtinCall(name, a.type, a);
const primOp2 = (name) => (a, b) => builtinCall(name, a.type, a, b);
const primOp3 = (name) => (a, b, c) => builtinCall(name, a.type, a, b, c);
/**
 * Returns normalized version of given vector.
 *
 * @param v -
 */
export const normalize = (v) => builtinCall("normalize", v.type, v);
/**
 * Returns length / magnitude of given vector.
 *
 * @param v -
 */
export const length = (v) => builtinCall("length", "float", v);
export const distance = (a, b) => builtinCall("distance", "float", a, b);
/**
 * Returns dot product of given vectors.
 *
 * @param a -
 * @param b -
 */
export const dot = (a, b) => builtinCall("dot", "float", a, b);
/**
 * Returns cross product of given 3D vectors.
 *
 * @param a -
 * @param b -
 */
export const cross = (a, b) => builtinCall("cross", a.type, a, b);
export const reflect = (i, n) => builtinCall("reflect", i.type, i, n);
export const refract = (i, n, ior) => builtinCall("refract", i.type, i, n, ior);
export const faceForward = (i, n, nref) => builtinCall("faceForward", i.type, i, n, nref);
export const min = primOp2("min");
export const max = primOp2("max");
export const clamp = primOp3("clamp");
export const step = primOp2("step");
export const smoothstep = primOp3("smoothstep");
export const radians = primOp1("radians");
export const degrees = primOp1("degrees");
export const cos = primOp1("cos");
export const sin = primOp1("sin");
export const tan = primOp1("tan");
export const acos = primOp1("acos");
export const asin = primOp1("asin");
export function atan(a, b) {
    const f = b
        ? builtinCall("atan", a.type, a, b)
        : builtinCall("atan", a.type, a);
    b && (f.info = "nn");
    return f;
}
export const pow = primOp2("pow");
export const exp = primOp1("exp");
export const log = primOp1("log");
export const exp2 = primOp1("exp2");
export const log2 = primOp1("log2");
export const sqrt = primOp1("sqrt");
export const inversesqrt = primOp1("inversesqrt");
export const abs = primOp1("abs");
export const sign = primOp1("sign");
export const floor = primOp1("floor");
export const ceil = primOp1("ceil");
export const fract = primOp1("fract");
export const powf = (x, y) => pow(x, matchingPrimFor(x, y));
export function mod(a, b) {
    const f = builtinCall("mod", a.type, a, b);
    b.type === "float" && (f.info = "n");
    return f;
}
/**
 * Important: Currently unsupported by JS target.
 *
 * TODO add additional metadata for JS (`b` is an output var), issue #96
 *
 * @param a -
 * @param b -
 */
// prettier-ignore
export const modf = (a, b) => builtinCall("modf", a.type, a, b);
export function mix(a, b, c) {
    const f = builtinCall("mix", a.type, a, b, c);
    c.type === "float" && (f.info = "n");
    return f;
}
// prettier-ignore
export const matrixCompMult = (a, b) => builtinCall("matrixCompMult", a.type, a, b);
export function isnan(a) {
    return builtinCall("isnan", matchingBoolType(a), a);
}
export function isinf(a) {
    return builtinCall("isinf", matchingBoolType(a), a);
}
