import { illegalArgs } from "@thi.ng/errors/illegal-arguments";
import { isVec } from "../ast/checks.js";
import { builtinCall } from "../ast/function.js";
const texRetType = (sampler) => {
    const t = sampler.type[0];
    const shadow = sampler.type.indexOf("Shadow") > 0;
    return t === "s"
        ? shadow
            ? "float"
            : "vec4"
        : t === "i"
            ? shadow
                ? "int"
                : "ivec4"
            : t === "u"
                ? shadow
                    ? "uint"
                    : "uvec4"
                : illegalArgs(`unknown sampler type ${sampler.type}`);
};
const $call = (name, sampler, args, bias) => {
    const f = bias
        ? builtinCall(name, texRetType(sampler), sampler, ...args, bias)
        : builtinCall(name, texRetType(sampler), sampler, ...args);
    !isVec(f) && (f.info = "n");
    return f;
};
// prettier-ignore
export function textureSize(sampler, lod) {
    return builtinCall("textureSize", /(3D|Array)/.test(sampler.type) ? "ivec3" : "ivec2", sampler, lod);
}
// prettier-ignore
export function texture(sampler, uv, bias) {
    return $call("texture", sampler, [uv], bias);
}
// prettier-ignore
export function textureProj(sampler, uv, bias) {
    return $call("textureProj", sampler, [uv], bias);
}
// prettier-ignore
export function textureLod(sampler, uv, lod) {
    return $call("textureLod", sampler, [uv, lod]);
}
// prettier-ignore
export function textureOffset(sampler, uv, off, bias) {
    return $call("textureOffset", sampler, [uv, off], bias);
}
// prettier-ignore
export function texelFetch(sampler, uv, lod) {
    return $call("texelFetch", sampler, [uv, lod]);
}
// prettier-ignore
export function texelFetchOffset(sampler, uv, lod, offset) {
    return $call("texelFetchOffset", sampler, [uv, lod, offset]);
}
// prettier-ignore
export function textureProjOffset(sampler, uv, off, bias) {
    return $call("textureProjOffset", sampler, [uv, off], bias);
}
// prettier-ignore
export function texelLodOffset(sampler, uv, lod, offset) {
    return $call("texelLodOffset", sampler, [uv, lod, offset]);
}
// prettier-ignore
export function textureProjLod(sampler, uv, lod) {
    return $call("textureProjLod", sampler, [uv, lod]);
}
// prettier-ignore
export function texelProjLodOffset(sampler, uv, lod, offset) {
    return $call("texelProjLodOffset", sampler, [uv, lod, offset]);
}
// prettier-ignore
export function textureGrad(sampler, uvw, dx, dy) {
    return $call("textureGrad", sampler, [uvw, dx, dy]);
}
// prettier-ignore
export function textureGradOffset(sampler, uvw, dx, dy, offset) {
    return $call("textureGradOffset", sampler, [uvw, dx, dy, offset]);
}
// prettier-ignore
export function textureProjGrad(sampler, uvw, dx, dy) {
    return $call("textureProjGrad", sampler, [uvw, dx, dy]);
}
// prettier-ignore
export function textureProjGradOffset(sampler, uvw, dx, dy, offset) {
    return $call("textureProjGradOffset", sampler, [uvw, dx, dy, offset]);
}
export const dFdx = (sym) => builtinCall("dFdx", sym.type, sym);
export const dFdy = (sym) => builtinCall("dFdy", sym.type, sym);
export const fwidth = (sym) => builtinCall("fwidth", sym.type, sym);
